#!/usr/bin/env python3

import sys, os

if len(sys.argv) != 3:
    print(sys.argv[0], '<namespace>', '<output dir>')

name = sys.argv[1]
odir = sys.argv[2]

with open(os.path.join(odir, name + '.h'), 'w') as f:
    f.write('int func();\n')
with open(os.path.join(odir, name + '.sh'), 'w') as f:
    f.write('#!/bin/bash')



# SPDX-License-Identifier: Apache-2.0
# Copyright 2021 The Meson development team

import pickle
from pathlib import Path
from .generatorbase import GeneratorBase
from .model import ReferenceManual

class GeneratorPickle(GeneratorBase):
    def __init__(self, manual: ReferenceManual, outpath: Path) -> None:
        self.out = outpath
        super().__init__(manual)

    def generate(self) -> None:
        self.out.write_bytes(pickle.dumps(self.manual))



# SPDX-License-Identifier: Apache-2.0
# Copyright 2021 The Meson development team
from __future__ import annotations

from pathlib import Path
import json
import re

from .generatorbase import GeneratorBase
from . import jsonschema as J
from .model import (
	ReferenceManual,
	Function,
	Object,
	Type,

	PosArg,
	VarArgs,
	Kwarg,
)

import typing as T

OBJ_MAP = {
	'build_machine': 'machine',
	'build_tgt': 'build_target',
	'cfg_data': 'configuration_data',
	'cmake': None,
	'cmake_options': None,
	'custom_idx': None,
	'custom_tgt': 'custom_target',
	'dep': 'dependency',
	'env': 'environment',
	'feature': 'feature_opt',
	'int': 'number',
	'list': 'array',
	'runresult': 'run_result',
	'str': 'string',
}

class GeneratorC(GeneratorBase):
	def __init__(self, manual: ReferenceManual, out: Path, enable_modules: bool) -> None:
		super().__init__(manual)
		self.out = out
		self.enable_modules = enable_modules

	def _generate_type(self, typ: Type) -> T.List[J.Type]:
		return [
			{
				'obj': x.data_type.name,
				'holds': self._generate_type(x.holds) if x.holds else [],
			}
			for x in typ.resolved
		]

	def _generate_type_str(self, typ: Type) -> str:
		# Remove all whitespaces
		return re.sub(r'[ \n\r\t]', '', typ.raw)

	def _generate_arg(self, arg: T.Union[PosArg, VarArgs, Kwarg], isOptarg: bool = False) -> J.Argument:
		return {
			'name': arg.name,
			'description': arg.description,
			'since': arg.since if arg.since else None,
			'deprecated': arg.deprecated if arg.deprecated else None,
			'type': self._generate_type(arg.type),
			'type_str': self._generate_type_str(arg.type),
			'required': arg.required if isinstance(arg, Kwarg) else not isOptarg and not isinstance(arg, VarArgs),
			'default': arg.default if isinstance(arg, (PosArg, Kwarg)) else None,
			'min_varargs': arg.min_varargs if isinstance(arg, VarArgs) and arg.min_varargs > 0 else None,
			'max_varargs': arg.max_varargs if isinstance(arg, VarArgs) and arg.max_varargs > 0 else None,

			# Not yet supported
			'notes': [],
			'warnings': [],
		}

	def _cstr(self, s: str) -> str:
		s = s.replace('\\', '\\\\')
		s = s.replace('\n', '\\n')
		s = s.replace('"', '\\"')
		return f'"{s}"'

	def _wrap(self, s):
		return f"{{ {s} }}"

	def _cstr_struct(self, d):
		elems = []
		for k, v in d.items():
			if v is True:
				s = "true"
			elif v is False:
				s = "false"
			elif type(v) is int:
				s = str(v)
			else:
				s = self._cstr(v)

			elems.append(f".{k} = {s}")

		return self._wrap(', '.join(elems))

	def _generate_function(self, func: Function) -> str:
		posargs = []
		kwargs = []

		for arg in func.posargs:
			posargs.append({
				'common.name': arg.name,
				'common.description': arg.description,
				'common.type': self._generate_type_str(arg.type),
			})

		for arg in func.optargs:
			posargs.append({
				'common.name': arg.name,
				'common.description': arg.description,
				'common.type': self._generate_type_str(arg.type),
				'optional': True,
			})

		if func.varargs:
			posargs.append({
				'common.name': func.varargs.name,
				'common.description': func.varargs.description,
				'common.type': self._generate_type_str(func.varargs.type),
				'glob': True,
			})

		for arg in self.sorted_and_filtered(list(func.kwargs.values())):
			kwargs.append({
				'common.name': arg.name,
				'common.description': arg.description,
				'optional': True,
			})

		return {
			'common.name': func.name,
			'common.description': func.description,
			'common.type': self._generate_type_str(func.returns),
			'posargs_len': len(posargs),
			'kwargs_len': len(kwargs),
		}, posargs, kwargs

		#	 'since': func.since if func.since else None,
		#	 'deprecated': func.deprecated if func.deprecated else None,
		#	 'notes': func.notes,
		#	 'warnings': func.warnings,
		#	 'example': func.example if func.example else None,
		#	 'returns': self._generate_type(func.returns),
		#	 'returns_str': self._generate_type_str(func.returns),
		#	 'posargs': {x.name: self._generate_arg(x) },
		#	 'optargs': {x.name: self._generate_arg(x, True) for x in func.optargs},
		#	 'kwargs': {x.name: self._generate_arg(x) for x in self.sorted_and_filtered(list(func.kwargs.values()))},
		#	 'varargs': self._generate_arg(func.varargs) if func.varargs else None,
		#	 'arg_flattening': func.arg_flattening,
		# }

	def _generate_objects(self, obj: Object) -> J.Object:
		return {
			'name': obj.name,
			'description': obj.description,
			'since': obj.since if obj.since else None,
			'deprecated': obj.deprecated if obj.deprecated else None,
			'notes': obj.notes,
			'warnings': obj.warnings,
			'defined_by_module': obj.defined_by_module.name if obj.defined_by_module else None,
			'object_type': obj.obj_type.name,
			'is_container': obj.is_container,
			'example': obj.example if obj.example else None,
			'extends': obj.extends if obj.extends else None,
			'returned_by': [x.name for x in self.sorted_and_filtered(obj.returned_by)],
			'extended_by': [x.name for x in self.sorted_and_filtered(obj.extended_by)],
			'methods': {x.name: self._generate_function(x) for x in self.sorted_and_filtered(obj.methods)},
		}

	def generate(self) -> None:
		prefix = 'meson_doc_'
		text = ''

		posargs = []
		kwargs = []
		funcs = { '0': [] }

		def add_function(group_name, func):
			f, p, k = self._generate_function(func)
			f['posargs_start'] = len(posargs)
			f['kwargs_start'] = len(kwargs)
			funcs[group_name].append(self._cstr_struct(f))
			posargs.extend([self._cstr_struct(x) for x in p])
			kwargs.extend([self._cstr_struct(x) for x in k])

		for x in self.sorted_and_filtered(self.functions):
			add_function('0', x)

		for x in self.sorted_and_filtered(self.objects):
			if x.name in OBJ_MAP:
				o = OBJ_MAP[x.name]
			else:
				o = x.name

			if o is None:
				continue

			o = f'obj_{o}'

			for y in self.sorted_and_filtered(x.methods):
				if o not in funcs:
					funcs[o] = []
				add_function(o, y)

		text += f'struct {prefix}entry_arg {prefix}posargs[] = {{\n'
		for o in posargs:
			text += f'\t{o},\n'

		text += '};\n'

		text += f'struct {prefix}entry_arg {prefix}kwargs[] = {{\n'
		for o in kwargs:
			text += f'\t{o},\n'

		text += '};\n'

		for o in funcs:
			text += f'struct {prefix}entry_func {prefix}object_{o}[] = {{\n'

			for f in funcs[o]:
				text += '\t' + f + ',\n'

			text += '\t{ 0 },\n'

			text += '};\n\n'

		text += f'struct {prefix}entry_func *{prefix}root[obj_type_count] = {{\n'

		for o in funcs:
			text += f'\t[{o}] = {prefix}object_{o},\n'

		text += '};\n'

		self.out.write_text(
			text.lstrip(),
			encoding='utf-8',
		)

		# data: J.Root = {
		#	 'version_major': J.VERSION_MAJOR,
		#	 'version_minor': J.VERSION_MINOR,
		#	 'meson_version': self._extract_meson_version(),
		#	 'functions': {x.name: self._generate_function(x) for x in self.sorted_and_filtered(self.functions)},
		#	 'objects': {x.name: self._generate_objects(x) for x in self.sorted_and_filtered(self.objects)},
		#	 'objects_by_type': {
		#		 'elementary': [x.name for x in self.elementary],
		#		 'builtins': [x.name for x in self.builtins],
		#		 'returned': [x.name for x in self.returned],
		#		 'modules': {
		#			 x.name: [y.name for y in self.sorted_and_filtered(self.extract_returned_by_module(x))]
		#			 for x in self.modules
		#		 },
		#	 },
		# }

		# self.out.write_text(json.dumps(data), encoding='utf-8')



import re
from pathlib import Path

from .generatorbase import GeneratorBase
from .model import (
    ReferenceManual,
    Function,
    Object,
    PosArg,
    VarArgs,
    Kwarg,
    ObjectType,
)

import typing as T

class GeneratorTypes(GeneratorBase):
    def __init__(
        self, manual: ReferenceManual, out: Path, enable_modules: bool
    ) -> None:
        super().__init__(manual)
        self.out = out
        self.enable_modules = enable_modules

    def _parse_type(self, t, in_container=False):
        parsed = []
        name = ""
        i = 0
        while i < len(t):
            c = t[i]
            if c == '[':
                (n, sub) = self._parse_type(t[i+1:], in_container=True)
                parsed.append((name, sub,))
                name = ""
                i += n + 1
                continue
            elif c == ']':
                if name:
                    parsed.append(name)
                return (i+1, parsed)
            elif c == ' ':
                i += 1
                continue
            elif c == '|':
                if name:
                    parsed.append(name)
                name = ""
            else:
                name += c

            i += 1

        if name:
            parsed.append(name)
        return (i, parsed)

    def parse_type(self, t):
        (_, parsed) = self._parse_type(t)
        return parsed

    def assemble_type(self, t):
        if type(t) is list:
            def sort_func(v):
                assert type(v) is not list

                if type(v) is tuple:
                    return v[0]
                else:
                    return v

            t.sort(key=sort_func)
            return "|".join(self.assemble_type(x) for x in t)
        elif type(t) is tuple:
            return t[0] + '[' + self.assemble_type(t[1]) + ']'
        else:
            return t

    def print_types(self, args):
        args = [args] if not isinstance(args, list) else args

        res = []
        for arg in args:
            t = self.assemble_type(self.parse_type(arg.type.raw))
            res.append(f"    {t}")

        return res


    def function_name(self, f: Function, o: Object = None) -> str:
        name = ""
        if o is not None:
            name += f"{o.name}."

        name += f.name
        return name

    def generate_function(self, f: Function, obj: Object = None):
        res = []
        res.append(self.function_name(f, obj))

        if f.posargs:
            res.append('  posargs:')
            res.extend(self.print_types(f.posargs))

        if f.varargs:
            res.append('  varargs:')
            res.extend(self.print_types(f.varargs))

        if f.optargs:
            res.append('  optargs:')
            res.extend(self.print_types(f.optargs))

        kwargs = self.sorted_and_filtered(list(f.kwargs.values()))
        if kwargs:
            res.append('  kwargs:')
        for kwarg in kwargs:
            k = kwarg.name
            t = self.assemble_type(self.parse_type(kwarg.type.raw))
            res.append(f"    {k}: {t}")

        res.append('  returns:')
        res.append('    ' + self.assemble_type(self.parse_type(f.returns.raw)))

        return res

    def generate(self):
        lines = []

        for f in self.sorted_and_filtered(self.functions):
            lines.extend(self.generate_function(f, None))

        for obj in self.sorted_and_filtered(self.objects):
            for f in self.sorted_and_filtered(obj.methods):
                if not self.enable_modules and (obj.obj_type == ObjectType.MODULE or obj.defined_by_module is not None):
                    continue

                lines.extend(self.generate_function(f, obj))

        self.out.write_text("\n".join(lines) + "\n", encoding="utf-8")



#!/usr/bin/env python3


import sys

plain_arg = sys.argv[1]
_, tail = plain_arg.split(':', 1)
filename, _ = tail.rsplit(':', 1)
try:
    with open(filename, 'rb') as f:
        content = f.read()
except FileNotFoundError:
    print('Could not open file. Missing dependency?')
    sys.exit(1)
print('File opened, pretending to send it somewhere.')
print(len(content), 'bytes uploaded')



import typing as T


def listify(item: T.Any, flatten: bool = True) -> T.List[T.Any]:
    """
    Returns a list with all args embedded in a list if they are not a list.
    This function preserves order.
    @flatten: Convert lists of lists to a flat list
    """
    if not isinstance(item, list):
        return [item]
    result = []  # type: T.List[T.Any]
    for i in item:
        if flatten and isinstance(i, list):
            result += listify(i, flatten=True)
        else:
            result.append(i)
    return result


class MesonException(Exception):
    """Exceptions thrown by Meson"""

    def __init__(
        self,
        *args: object,
        file: T.Optional[str] = None,
        lineno: T.Optional[int] = None,
        colno: T.Optional[int] = None
    ):
        super().__init__(*args)
        self.file = file
        self.lineno = lineno
        self.colno = colno


def relpath(path: str, start: str) -> str:
    # On Windows a relative path can't be evaluated for paths on two different
    # drives (i.e. c:\foo and f:\bar).  The only thing left to do is to use the
    # original absolute path.
    try:
        return os.path.relpath(path, start)
    except (TypeError, ValueError):
        return path



# SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
# SPDX-License-Identifier: GPL-3.0-only

#!/usr/bin/env python3

import glob
import json
import os
import shlex
import stat
import sys


def die(msg):
    print(msg)
    sys.exit(1)


def write_check_script(dat, path):
    s = "#!/bin/sh\nset -eux\n"

    for e in dat:
        if e["type"] == "dir":
            test = "-d"
        elif e["type"] == "file":
            test = "-f"
        else:
            # TODO: handle other file types
            continue

        file = shlex.quote(e["file"])
        s += f'test {test} "$DESTDIR"/{file}\n'

    with open(path, "w") as f:
        f.write(s)
    os.chmod(path, 0o755)


def main(argc, argv):
    if argc < 2:
        die(f"usage: {argv[0]} <path/to/tests>")

    for d in glob.glob(f"{argv[1]}/*"):
        test_json = d + "/test.json"
        check_script = d + "/check.sh"

        if not os.access(test_json, os.F_OK):
            continue

        with open(test_json) as f:
            dat = json.load(f)

        if "installed" in dat:
            write_check_script(dat["installed"], check_script)


if __name__ == "__main__":
    main(len(sys.argv), sys.argv)



#! /usr/bin/env python3

import json
import sys
import os

cc = None
output = None

# Only the ninja backend produces compile_commands.json
if sys.argv[1] == "ninja":
    if os.path.exists("compile_commands.json"):
        with open("compile_commands.json", "r") as f:
            cc = json.load(f)
        output = set((x["output"] for x in cc))

for obj in sys.argv[2:]:
    if not os.path.exists(obj):
        sys.exit(f"File {obj} not found.")
    if output and obj not in output:
        sys.exit(f"File {obj} not in output.")

    print("Verified", obj)



#!/usr/bin/env python3

import sys, os

if __name__ == '__main__':
    if len(sys.argv) < 4:
        print(sys.argv[0], 'n output objects...')
        sys.exit(1)
    if len(sys.argv) != int(sys.argv[1]) + 3:
        print(f'expected {sys.argv[1]} objects, got {len(sys.argv) - 3}')
        sys.exit(1)
    for i in sys.argv[3:]:
        print('testing', i)
        if not os.path.exists(i):
            sys.exit(1)
    with open(sys.argv[2], 'wb') as out:
        pass



# SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
# SPDX-License-Identifier: GPL-3.0-only

import sys
import itertools

from version_info import MuonVersion

function_support_notes = {
        "subproject": """
            <a href=\"https://git.sr.ht/~lattis/muon/tree/master/item/doc/differences.md#nested-subproject-promotion\">
                muon does not perform subproject promotion
            </a>
        """,
}

def parse(file):
    sigs = {}

    with open(file, "r") as f:
        argi = {
            "posargs:": 0,
            "varargs:": 1,
            "optargs:": 2,
            "kwargs:": 3,
            "returns:": 4,
        }

        fname = None
        args = [[], [], [], [], []]
        curargs = None
        extension = False

        for line in f.read().strip().split("\n"):
            if line[0] != " ":
                if fname:
                    sigs[fname] = Sig(fname, *args, extension)
                    fname = None
                    args = [[], [], [], [], []]
                    curargs = None
                    extension = False

                l = line.split(":")
                if len(l) > 1 and l[0] == "extension":
                    extension = True
                fname = l[-1]
            elif line[2] != " ":
                curargs = argi[line.strip()]
            else:
                v = line.strip()
                if curargs == 3:  # kwargs
                    v = v.split(": ")

                args[curargs].append(v)

        if fname:
            sigs[fname] = Sig(fname, *args, extension)

    return sigs


class Sig:
    def empty(name):
        return Sig(name, [], [], [], [], [], False)

    def __init__(self, name, posargs, varargs, optargs, kwargs, returns, extension):
        self.name = name
        self.posargs = posargs
        self.varargs = varargs
        self.optargs = optargs
        self.kwargs = {self.esc(k): v for k, v in self.normalize_kw(kwargs)}
        if returns:
            self.returns = returns[0]
        else:
            self.returns = returns
        self.extension = extension

    def normalize_kw(self, kwargs):
        applies_to = [
            "executable",
            "library",
            "static_library",
            "shared_library",
            "shared_module",
            "build_target",
            "both_libraries",
        ]

        if self.name not in applies_to:
            return kwargs

        prefixes = ["c_", "cpp_", "objc_"]
        delete = []
        add = {}
        for kw, v in kwargs:
            for pre in prefixes:
                if kw.startswith(pre):
                    delete.append(kw)
                    add["<lang>_" + kw[len(pre) :]] = v
                    break

        add = [[k, v] for k, v in add.items()]

        return [x for x in kwargs if x[0] not in delete] + add

    def esc(self, c):
        return str(c).replace("<", "&#60;").replace(">", "&#62;")

    def arg_count(self):
        return (
            len(self.posargs) + len(self.varargs) + len(self.optargs) + len(self.kwargs)
        )


muon = parse(sys.argv[1])
meson = parse(sys.argv[2])


class Table:
    def __init__(self, thead):
        self.thead = thead
        self.rows = []

    def __add__(self, other):
        self.rows.append(other)
        return self

    def html(self):
        return (
            "<table><thead>"
            + self.thead.html()
            + "</thead><tbody>"
            + "".join([r.html() for r in self.rows])
            + "</tbody></table>"
        )


class Row:
    def __init__(self, *cols):
        self.cols = cols

    def html(self):
        return (
            "<tr>" + "".join(["<td>" + str(c) + "</td>" for c in self.cols]) + "</tr>"
        )


class FuncInfo:
    def __init__(self, name):
        self.name = name
        self.me = None
        self.mu = None


def _parse_type(t, in_container=False):
    parsed = []
    name = ""
    i = 0
    while i < len(t):
        c = t[i]
        if c == "[":
            (n, sub) = _parse_type(t[i + 1 :], in_container=True)
            parsed.append(
                (
                    name,
                    sub,
                )
            )
            name = ""
            i += n + 1
            continue
        elif c == "]":
            if name:
                parsed.append(name)
            return (i + 1, parsed)
        elif c == " ":
            i += 1
            continue
        elif c == "|":
            if name:
                parsed.append(name)
            name = ""
        else:
            name += c

        i += 1

    if name:
        parsed.append(name)
    return (i, parsed)


def parse_type(t):
    if t is None:
        return []

    (_, parsed) = _parse_type(t)
    return parsed


# Attempt to detect if a signature indicates listification and remove the
# duplicate part
def normalize_listify(tp):
    if not tp:
        return tp

    nl = []
    l = set()

    for t in tp:
        if type(t) is tuple:
            if t[0] == "list":
                l = t[1]
            else:
                return tp
        else:
            nl.append(t)

    if set(nl) == set(l):
        return [("list", nl)]
    else:
        return tp


def assemble_type(t):
    if type(t) is list:

        def sort_func(v):
            assert type(v) is not list

            if type(v) is tuple:
                return v[0]
            else:
                return v

        t.sort(key=sort_func)
        return " | ".join(assemble_type(x) for x in t)
    elif type(t) is tuple:
        return t[0] + "[" + assemble_type(t[1]) + "]"
    else:
        return t


def neutral(text):
    return f'<span class="neutral">{text}</span>'


def positive(text):
    return f'<span class="positive">{text}</span>'


def negative(text):
    return f'<span class="negative">{text}</span>'

def warning(text):
    return f'<span class="warning">{text}</span>'


def tdiff(a, b):
    s = []
    for t in a:
        if type(t) is tuple:
            other = None
            for u in b:
                if type(u) is tuple and t[0] == u[0]:
                    other = u[1]
                    break

            if other:
                s.append(neutral(f"{t[0]}[{tdiff(t[1], other)}]"))
            else:
                s.append(positive(f"{t[0]}[{assemble_type(t[1])}]"))
        else:
            if t in b:
                s.append(neutral(t))
            else:
                s.append(positive(t))

    return " | ".join(s)


def normalize_types(tp):
    conts = []
    ntp = set()
    for t in tp:
        if type(t) is tuple:
            sub = normalize_types(t[1])
            if not sub:
                ntp |= set([t[0]])
            else:
                conts.append((t[0], sub))
        else:
            ntp |= set(
                {
                    "custom_idx": ["file"],
                    "extracted_obj": ["file"],
                    "null": [],
                    "tgt": ["build_tgt", "custom_tgt", "both_libs"],
                    "lib": ["build_tgt", "both_libs"],
                    "exe": ["build_tgt"],
                }.get(t, [t])
            )

    def sort_func(v):
        assert type(v) is not list

        if type(v) is tuple:
            return v[0]
        else:
            return v

    return sorted(list(ntp) + conts, key=sort_func)


def typecomp(mu, me):
    a = normalize_listify(normalize_types(parse_type(mu)))
    b = normalize_listify(normalize_types(parse_type(me)))
    return (tdiff(a, b), tdiff(b, a))

def footnote(footnotes, footnote):
    if footnote not in footnotes:
        footnotes.append(footnote)

    index = footnotes.index(footnote) + 1
    return f"<sup><a href=\"#footnote{index}\">{index}</a></sup>"


func_tbl = Table(Row("function", "status", "muon return", "meson return"))
arg_tbls = []
footnotes = []

all_funcs = set(muon.keys()) | set(meson.keys())
methods = set([f for f in all_funcs if "." in f])
kernel = all_funcs - methods

for f in sorted(kernel) + sorted(methods):
    if f in ["custom_idx.full_path", "custom_tgt.[index]"]:
        continue

    r = FuncInfo(f)
    if f in muon:
        flink = f'<a href="#{f}">{f}</a>'
        r.mu = muon[f]
    else:
        flink = f

    if f in meson:
        r.me = meson[f]

    support = positive("supported")
    notes = ""

    if f not in muon:
        r.mu = Sig.empty(f)
        support = negative("unsupported")
    elif f not in meson:
        r.me = Sig.empty(f)
        if muon[f].extension:
            support = "muon extension"
        else:
            support = warning("supported")
            notes = footnote(footnotes, "There is no type information for this function in the meson yaml documentation, therefore a comparison cannot be made.")

    if f in function_support_notes:
        support = warning("supported")
        notes += footnote(footnotes, function_support_notes[f])

    if notes:
        support += '<div class="footnotes">' + notes + '</div>'

    support = f'<div class="support_cell">{support}</div>'

    func_tbl += Row(flink, support, *typecomp(r.mu.returns, r.me.returns))

    t = Table(Row("kind", "keyword", "muon type", "meson type"))

    for k in ["posarg", "vararg", "optarg"]:
        for a, b in itertools.zip_longest(
            getattr(r.mu, k + "s"), getattr(r.me, k + "s")
        ):
            t += Row(k, "", *typecomp(a, b))

    for k in sorted(set(r.me.kwargs.keys()) | set(r.mu.kwargs.keys())):
        t1 = None
        t2 = None

        if k in r.mu.kwargs:
            t1 = r.mu.kwargs[k]

        if k in r.me.kwargs:
            t2 = r.me.kwargs[k]

        t += Row("kwarg", k, *typecomp(t1, t2))

    if f in muon:
        arg_tbls.append((r, t))

module_tbl = Table(Row("module", "status"))
for m in ["fs", "keyval", "pkgconfig", "sourceset", "i18n"]:
    module_tbl += Row(m, positive("supported"))

for m in ["python3", "python"]:
    module_tbl += Row(m, "partial")

for m in [
    "cmake",
    "dlang",
    "gnome",
    "hotdoc",
    "java",
    "modtest",
    "qt",
    "qt4",
    "qt5",
    "qt6",
    "unstable-cuda",
    "unstable-external_project",
    "unstable-icestorm",
    "unstable-rust",
    "unstable-simd",
    "unstable-wayland",
    "windows",
]:
    module_tbl += Row(m, negative("unsupported"))

print(
    """<!DOCTYPE html>
<html><head>
<link rel="stylesheet" href="status.css" />
<title>muon implementation status</title>
<link rel="icon" type="image/svg+xml" href="muon_logo.svg">
</head><body><div class="wrapper">
"""
)

print('<div class="item">')
print("<h1>Version</h1>")
print(f"<p>muon version {MuonVersion.version}-{MuonVersion.vcs_tag}</p>")
print(f"<p>meson compat version {MuonVersion.meson_compat}</p>")
print("</div>")

print('<div class="item">')
print(f"<h1>Modules</h1>")
print(module_tbl.html())
print(f"<h1>Functions and methods</h1>")
print(func_tbl.html())
print("</div>")
for r, at in arg_tbls:
    print("<hr>")
    print('<div class="item">')
    print(f'<h1 id="{r.name}">{r.name}</h1>')
    if at.rows:
        print(at.html())
    else:
        print("no arguments")
    print("</div>")

print("<hr>")
print('<div class="item">')
print('<h1 id="footnotes">Footnotes</h1>')
print("<ol>")
for index in range(len(footnotes)):
    print(f'<li id="footnote{index + 1}">{footnotes[index]}</li>')
print("</ol>")
print('</div>')

print("</div></body>")



#!/usr/bin/env python3

import os, sys
from pathlib import Path

assert 'MESON_SOURCE_ROOT' in os.environ
assert 'MESON_BUILD_ROOT' in os.environ
assert 'MESON_SUBDIR' in os.environ
# different than meson: MESONINTROSPECT is not set
# assert 'MESONINTROSPECT' in os.environ
assert 'MY_ENV' in os.environ

# Environment has absolute paths and argv has relative paths when using ninja
# backend and absolute paths when using vs backend. What matters is once
# resolved they point to same location.
env_source_root = Path(os.environ['MESON_SOURCE_ROOT']).resolve()
env_build_root = Path(os.environ['MESON_BUILD_ROOT']).resolve()
env_current_source_dir = Path(env_source_root, os.environ['MESON_SUBDIR']).resolve()

print(sys.argv)
argv_paths = [Path(i).resolve() for i in sys.argv[1:]]
source_root, build_root, current_source_dir = argv_paths

print(f'{source_root} == {env_source_root}')
assert source_root == env_source_root
print(f'{build_root} == {env_build_root}')
assert build_root == env_build_root
print(f'{current_source_dir} == {env_current_source_dir}')
assert current_source_dir == env_current_source_dir
